'use strict';

var highlight_js = require('highlight.js');
var colorette = require('colorette');

/* eslint-disable no-console, no-underscore-dangle */

function colorize(part) {
  switch (part.kind) {
    case 'keyword':
    case 'literal':
      return colorette.gray(part.children[0].toUpperCase())
    case 'number':
    case 'string':
      return colorette.yellowBright(part.children[0])
    default:
      return part.children[0]
  }
}

function knexQueryLogger(knex) {
  const executedQueries = {};

  function highlightQuery({ bindings, sql }) {
    return highlight_js.highlight('sql', knex.raw(sql, bindings).toString()).emitter.rootNode.children
      .map((part) => ((typeof part === 'string') ? colorette.gray(part) : colorize(part)))
      .join('')
  }

  knex
    .on('query', ({ __knexQueryUid, bindings, sql }) => {
      executedQueries[__knexQueryUid] = {
        bindings,
        sql,
        startTime: process.hrtime.bigint()
      };
    })
    .on('query-error', (_, { __knexQueryUid }) => {
      const query = executedQueries[__knexQueryUid];

      console.log(`${colorette.red('failed')} | ${highlightQuery(query)}`);

      delete executedQueries[__knexQueryUid];
    })
    .on('query-response', (_, { __knexQueryUid }) => {
      const { startTime, ...query } = executedQueries[__knexQueryUid];
      const endTime = process.hrtime.bigint();
      const duration = (Number(endTime - startTime) / 1e6).toFixed(2);

      if (query.sql) {
        console.log(highlightQuery(query));
        console.log(colorette.white(`${duration} ms`));
      } else {
        console.log(colorette.red('Missing sql'));
      }

      delete executedQueries[__knexQueryUid];
    });
}

module.exports = knexQueryLogger;
